;========================================;
;	  DRIVER for MOTOR DC 	 	 ;
;----------------------------------------;
;File Name :	DCMOTOR.INC		 ;
;Date	   :	04/01/03		 ;
;========================================;
		DSEG
CommandI2C	EQU	BufferOut0
GateTime	EQU	BufferOut1
PWM1		EQU	BufferOut2
PWM2		EQU	BufferOut3
DCControl	EQU	BufferOut4

InputH1		EQU	BufferIn0
InputL1		EQU	BufferIn1
;InputH2		EQU	BufferIn2
;InputL2		EQU	BufferIn3
;BrakeH1		EQU	BufferIn4
;BrakeL1		EQU	BufferIn5
;BrakeH2		EQU	BufferIn6
;BrakeL2		EQU	BufferIn7

		CSEG
;-----------------------------------------
;DC Motor Initial I2C Routine
;Input  : AddressI2C,GateTime,PWM1,PWM2,
;	  DCControl
;Output : FAck
;-----------------------------------------
DCInit:
		PUSH	ACC
		LCALL	StartCon
		MOV	A,AddressI2C
		ANL	A,#0FEh
		CLR	FAck
		ACALL	MTx
		JNB	FAck,EODCInit
		MOV	A,#00H
		CLR	FAck
		ACALL	MTx
		JNB	FAck,EODCInit
		MOV	A,GateTime
		CLR	FAck
		ACALL	MTx
		JNB	FAck,EODCInit
		MOV	A,PWM1
		CLR	FAck
		ACALL	MTx
		JNB	FAck,EODCInit
		MOV	A,PWM2
		CLR	FAck
		ACALL	MTx
		JNB	FAck,EODCInit
		MOV	A,DCControl
		CLR	FAck
		ACALL	MTx
EODCInit:	
		ACALL	StopCon
		POP	ACC
		RET

;-----------------------------------------
;Transmit Data 1 Byte To I2C
;Input  : AddressI2C,Command,BufferOut1
;Output : FAck
;-----------------------------------------
SendI2C:
		PUSH	ACC
		ACALL	StartCon
		MOV	A,AddressI2C
		ANL	A,#0FEh
		CLR	FAck
		ACALL	MTx
		JNB	FAck,EOSendI2C
		MOV	A,CommandI2C
		CLR	FAck
		ACALL	MTx
		JNB	FAck,EOSendI2C
		MOV	A,BufferOut1
		CLR	FAck
		ACALL	MTx
EOSendI2C:	
		ACALL	StopCon
		POP	ACC
		RET

;-----------------------------------------
;Set GateTime Value
;Input  : AddressI2C,GateTime
;Output : FAck 
;-----------------------------------------
SetGateTime:
		MOV	CommandI2C,#10H
		MOV	BufferOut1,GateTime
		ACALL	SendI2C
		RET

;-----------------------------------------
;Set PWM Motor 1 Value
;Input  : AddressI2C,PWM1
;Output : FAck
;-----------------------------------------
SetPWM1:
		PUSH	ACC
		MOV	A,BufferOut1
		MOV	CommandI2C,#20H
		MOV	BufferOut1,PWM1
		ACALL	SendI2C
		MOV	BufferOut1,A
		POP	ACC
		RET

;-----------------------------------------
;Set PWM Motor 2 Value
;Input  : AddressI2C,PWM2
;Output : FAck
;-----------------------------------------
;SetPWM2:
;		PUSH	ACC
;		MOV	A,BufferOut1
;		MOV	CommandI2C,#30H
;		MOV	BufferOut1,PWM2
;		ACALL	SendI2C
;		MOV	BufferOut1,A
;		POP	ACC
;		RET

;-----------------------------------------
;Set Control DC Motor
;Input  : AddressI2C,DCControl
;Output : FAck
;-----------------------------------------
SetControl:
		PUSH	ACC
		MOV	A,BufferOut1
		MOV	CommandI2C,#40H
		MOV	BufferOut1,DCControl
		ACALL	SendI2C
		MOV	BufferOut1,A
		POP	ACC
		RET

;-----------------------------------------
;Get Data Input DC Motor 1
;Input  : AddressI2C
;Output : InputH1,InputL1
;-----------------------------------------
GetInput1:
		PUSH	ACC
		MOV	CommandI2C,#50H
		MOV	A,BufferOut1
		MOV	BufferOut1,#00H
		ACALL	SendI2C
		MOV	BufferOut1,A

		ACALL	StartCon
		MOV	A,AddressI2C
		ORL	A,#00000001B
		CLR	FAck
		ACALL	MTx
		JNB	FAck,EOGInput1
		CLR	FLB
		ACALL	MRx
		MOV	InputL1,A
		SETB	FLB
		ACALL	MRx
		MOV	InputH1,A
EOGInput1:	ACALL	StopCon
		POP	ACC
		RET

;-----------------------------------------
;Get Data Input DC Motor 2
;Input  : AddressI2C
;Output : InputH2,InputL2
;-----------------------------------------
;GetInput2:
;		PUSH	ACC
;		MOV	CommandI2C,#50H
;		MOV	A,BufferOut1
;		MOV	BufferOut1,#02H
;		ACALL	SendI2C
;		MOV	BufferOut1,A

;		ACALL	StartCon
;		MOV	A,AddressI2C
;		ORL	A,#00000001B
;		CLR	FAck
;		ACALL	MTx
;		JNB	FAck,EOGInput2
;		CLR	FLB
;		ACALL	MRx
;		MOV	InputL2,A
;		SETB	FLB
;		ACALL	MRx
;		MOV	InputH2,A
;EOGInput2:	ACALL	StopCon
;		POP	ACC
;		RET

;-----------------------------------------
;Brake DC Motor 1
;Input  : AddressI2C,BrakeH1,BrakeL1
;Output : InputH1,InputL1 
;	  Motor 1 stop (DCControl.0 = '1')
;-----------------------------------------
;Brake1:
;		PUSH	PSW
;		PUSH	ACC
;		PUSH	B
;		PUSH	03H
;		PUSH	04H
;		PUSH	07H
;		MOV	R7,GateTime
;		MOV	GateTime,#01H
;		ACALL	SetGateTime
;		MOV	A,DCControl
;		CPL	ACC.1
;		MOV	DCControl,A
;		ACALL	SetControl
;CekInput1:	
;		ACALL	DelayBrk
;		ACALL	GetInput1
;		MOV	A,InputH1
;		MOV	B,InputL1
;		MOV	R3,BrakeH1
;		MOV	R4,BrakeL1
;		ACALL	LTE16Bit
;		JB	F0,BrkCon1
;		AJMP	CekInput1
;BrkCon1:		
;		MOV	A,DCControl
;		SETB	ACC.0
;		CPL	ACC.1
;		MOV	DCControl,A
;		ACALL	SetControl
;		MOV	GateTime,R7
;		ACALL	SetGateTime
;		POP	07H
;		POP	04H
;		POP	03H
;		POP	B
;		POP	ACC
;		POP	PSW
;		RET
;-----------------------------------------
;Brake DC Motor 2
;Input  : AddressI2C,BrakeH2,BrakeL2
;Output : InputH2,InputL2
;	  Motor 2 stop (DCControl.2 = '1')
;-----------------------------------------
;Brake2:
;		PUSH	PSW
;		PUSH	ACC
;		PUSH	B
;		PUSH	03H
;		PUSH	04H
;		PUSH	07H
;		MOV	R7,GateTime
;		MOV	GateTime,#01H
;		ACALL	SetGateTime
;		MOV	A,DCControl
;		CPL	ACC.3
;		MOV	DCControl,A
;		ACALL	SetControl
;CekInput2:	
;		ACALL	DelayBrk
;		ACALL	GetInput2
;		MOV	A,InputH2
;		MOV	B,InputL2
;		MOV	R3,BrakeH2
;		MOV	R4,BrakeL2
;		ACALL	LTE16Bit
;		JB	F0,BrkCon2
;		AJMP	CekInput2
;BrkCon2:		
;		MOV	A,DCControl
;		SETB	ACC.2
;		CPL	ACC.3
;		MOV	DCControl,A
;		ACALL	SetControl
;		MOV	GateTime,R7
;		ACALL	SetGateTime
;		POP	07H
;		POP	04H
;		POP	03H
;		POP	B
;		POP	ACC
;		POP	PSW
;		RET
;-----------------------------------------
;Delay Routine for Brake
;-----------------------------------------
;DelayBrk:					;2
;		PUSH	06H			;2
;		PUSH	07H			;2
;		MOV	R6,#14			;1 (28)
;		MOV	R7,#0FFH		;1 x R6
;		DJNZ	R7,$			;2 x R7 x R6 
;		DJNZ	R6,$-4			;2 x R6		
;		POP	07H			;2
;		POP	06H			;2
;		RET				;2
;-------------------------------
;Check Less than or equal 16 bit
;Input  :       MSB LSB
;	  Op1 = A   B
;	  Op2 = R3  R4
;Output : F0=1 if Op1 <= Op2
;	  F0=0 if Op1 > Op2
;	  AB = AB-R3R4
;Remark : None
;-------------------------------
LTE16bit:	XCH	A,B		
		CLR	C
		SUBB	A,R4
		XCH	A,B
		SUBB	A,R3
		JC	LTE
		JNZ	Greater
		MOV	A,B
		JNZ	Greater
LTE:		SETB	F0
		AJMP	EOLTE16Bit
Greater:	CLR	F0
EOLTE16Bit:	RET